package cmdTemplate

import (
    "errors"
    "strings"
    "nexus-common/params"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type Edit struct {
    Name string
}

func (cmd *Edit)GetName() string {
    return cmd.Name
}

func (cmd *Edit)GetDesc() []string {
    return []string{
        "Edits one or more template's properties: name, access.",
        "Requires TPL_EDIT or TPL_EDIT_ANY user capability."}
}

func (cmd *Edit)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.GetName()+" ID [name=\"new name\"] [access=private/public]")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("ID      ID of the template to edit.")
    const usage string = `Parameters that can be changed (at least one must be specified):
name    Name of the template.
access  Access type, either "private" or "public".`
}

func (cmd *Edit)Run(args []string) int {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    argc := len(args)
    if argc < 2 {
        cmd.PrintUsage()
        return 1
    }

    p, err := cmd.parseArgs(args[1:])
    if err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }

    if p == nil {
        cmd.PrintUsage()
        return 1
    }

    tplID := args[0]
    statusCode := 0

    resp, err := client.R().SetBody(p).Put(host+"/templates/"+tplID)
    if err != nil {
        u.PrintlnErr("Failed editing template \""+tplID+"\": "+err.Error())
        statusCode = 1
    } else {
        if resp.IsSuccess() {
            u.Println("Successfully edited template \""+tplID+"\":")
            u.Println(resp.String())
        } else {
            u.PrintlnErr("Failed editing template \""+tplID+"\": "+resp.Status()+": "+resp.String())
            statusCode = 1
        }
    }

    return statusCode
}

func (cmd *Edit)parseArgs(args []string) (*params.TplEdit, error) {
    p := &params.TplEdit{}
    atLeastOneArg := false

    getStringVal := func(s string, prefix string) string {
        if strings.HasPrefix(s, prefix) {
            parts := strings.Split(s, prefix)
            return parts[1]
        }
        return ""
    }

    for _, arg := range args {
        s := getStringVal(arg, "name=")
        if s != "" {
            p.Name = s
            atLeastOneArg = true
            continue
        }
        s = getStringVal(arg, "access=")
        if s != "" {
            p.Access = s
            atLeastOneArg = true
            continue
        }
        return nil, errors.New("Invalid argument: "+arg)
    }

    if atLeastOneArg {
        return p, nil
    } else {
        return nil, nil
    }
}
